<?php

declare(strict_types=1);

namespace media_library_texts\Tests\media_library\Kernel;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Form\FormState;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\entity_test\Entity\EntityTest;
use Drupal\media\Entity\MediaType;

/**
 * Tests overriding texts in media library widget.
 *
 * @group media_library_texts
 */
class MediaLibraryWidgetTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'media',
    'media_library',
    'media_library_texts',
    'field',
    'filter',
    'image',
    'system',
    'views',
    'user',
    'entity_test',
  ];

  /**
   * An admin user.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $adminUser;

  /**
   * The base field definition.
   *
   * @var \Drupal\Core\Field\BaseFieldDefinition
   */
  protected BaseFieldDefinition $baseField;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->baseField = BaseFieldDefinition::create('entity_reference')
      ->setName('media')
      ->setSetting('target_type', 'media')
      ->setSetting('handler_settings', ['target_bundles' => ['test_type' => 'test_type']]);
    $this->container->get('state')->set('entity_test.additional_base_field_definitions', [
      'media' => $this->baseField,
    ]);
    $this->container->get('state')->set('entity_test_rev.additional_base_field_definitions', [
      'media' => $this->baseField,
    ]);

    $this->installEntitySchema('entity_test');
    $this->installEntitySchema('entity_test_rev');
    $this->installEntitySchema('user');
    $this->installConfig([
      'system',
      'image',
      'media',
      'media_library',
    ]);

    MediaType::create([
      'id' => 'test_type',
      'label' => 'Test type',
      'source' => 'image',
    ])->save();

    // Create user 1 so the test user doesn't bypass access control.
    $this->createUser();

    $this->adminUser = $this->createUser([
      'administer entity_test content',
      'view media',
    ]);
  }

  /**
   * Provide data to test overriding the add button label.
   *
   * @return array
   *   The data array with list of overridden button labels.
   */
  public function dataProviderTestButtonLabel(): array {
    return [
      'no_override' => [
        'label' => NULL,
        'expected' => 'Add media',
      ],
      'override' => [
        'label' => 'Add image',
        'expected' => 'Add image',
      ],
    ];
  }

  /**
   * Test the button label in the media library widget.
   *
   * @param string|null $label
   *   Given button label.
   * @param string $expected
   *   Expected button label.
   *
   * @dataProvider dataProviderTestButtonLabel
   */
  public function testButtonLabel(string|null $label, string $expected): void {
    $entity = EntityTest::create([
      'name' => 'sample entity',
    ]);
    $entity->save();

    $third_party_settings = [];

    if (!is_null($label)) {
      $third_party_settings = [
        'media_library_texts' => [
          'add_button_text' => $label,
        ],
      ];
    }
    $element = $this->buildWidgetForm(
      entity: $entity,
      third_party_settings: $third_party_settings,
    );

    $button_label = $element['open_button']['#value'];
    if ($button_label instanceof TranslatableMarkup) {
      $button_label = $button_label->render();
    }

    $this->assertEquals($button_label, $expected);
  }

  /**
   * Build the media library widget form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to build the form for.
   * @param array $third_party_settings
   *   (Optional) Additional settings used to build the widget form.
   *   Defaults to an empty array.
   * @param string $description
   *   (Optional) Description to use for the widget.
   *   Defaults to an empty string.
   *
   * @return array
   *   A built form array of the media library widget.
   */
  protected function buildWidgetForm(EntityInterface $entity, array $third_party_settings = [], string $description = '') {
    $form = [
      '#parents' => [],
    ];

    /** @var \Drupal\Core\Field\WidgetInterface $widget */
    $widget = $this->container->get('plugin.manager.field.widget')->createInstance('media_library_widget', [
      'field_definition' => $this->baseField,
      'settings' => [],
      'third_party_settings' => $third_party_settings,
    ]);

    $form_state = new FormState();

    // @phpstan-ignore-next-line
    $element = $widget->formElement($entity->media, 0, ['#description' => $description], $form, $form_state);

    $context = [
      'widget' => $widget,
      // @phpstan-ignore-next-line
      'items' => $entity->media,
    ];
    $this->container->get('module_handler')->alter('field_widget_single_element_form', $element, $form_state, $context);

    return $element;
  }

}
